@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "mTunnel" multicast access service
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // Helper routines to implement 'group sockets'
 // Implementation
 
@@ -35,7 +35,7 @@ extern "C" int initializeWinsockIfNecessary();
 netAddressBits SendingInterfaceAddr = INADDR_ANY;
 netAddressBits ReceivingInterfaceAddr = INADDR_ANY;
 
-static void socketErr(UsageEnvironment& env, char* errorMsg) {
+static void socketErr(UsageEnvironment& env, char const* errorMsg) {
 	env.setResultErrMsg(errorMsg);
 }
 
@@ -49,31 +49,25 @@ NoReuse::~NoReuse() {
   reuseFlag = 1;
 }
 
-int setupDatagramSocket(UsageEnvironment& env, Port port,
-#ifdef IP_MULTICAST_LOOP
-			Boolean setLoopback
-#else
-			Boolean
-#endif
-) {
+int setupDatagramSocket(UsageEnvironment& env, Port port) {
   if (!initializeWinsockIfNecessary()) {
     socketErr(env, "Failed to initialize 'winsock': ");
     return -1;
   }
-  
+
   int newSocket = socket(AF_INET, SOCK_DGRAM, 0);
   if (newSocket < 0) {
     socketErr(env, "unable to create datagram socket: ");
     return newSocket;
   }
-  
+
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEADDR,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEADDR) error: ");
     closeSocket(newSocket);
     return -1;
   }
-  
+
 #if defined(__WIN32__) || defined(_WIN32)
   // Windoze doesn't properly handle SO_REUSEPORT or IP_MULTICAST_LOOP
 #else
@@ -85,9 +79,9 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
     return -1;
   }
 #endif
-  
+
 #ifdef IP_MULTICAST_LOOP
-  const u_int8_t loop = (u_int8_t)setLoopback;
+  const u_int8_t loop = 1;
   if (setsockopt(newSocket, IPPROTO_IP, IP_MULTICAST_LOOP,
 		 (const char*)&loop, sizeof loop) < 0) {
     socketErr(env, "setsockopt(IP_MULTICAST_LOOP) error: ");
@@ -96,13 +90,15 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
   }
 #endif
 #endif
-  
+
   // Note: Windoze requires binding, even if the port number is 0
+  netAddressBits addr = INADDR_ANY;
 #if defined(__WIN32__) || defined(_WIN32)
 #else
   if (port.num() != 0 || ReceivingInterfaceAddr != INADDR_ANY) {
 #endif
-    MAKE_SOCKADDR_IN(name, ReceivingInterfaceAddr, port.num());
+    if (port.num() == 0) addr = ReceivingInterfaceAddr;
+    MAKE_SOCKADDR_IN(name, addr, port.num());
     if (bind(newSocket, (struct sockaddr*)&name, sizeof name) != 0) {
       char tmpBuffer[100];
       sprintf(tmpBuffer, "bind() error (port number: %d): ",
@@ -115,12 +111,12 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
 #else
   }
 #endif
-  
+
   // Set the sending interface for multicasts, if it's not the default:
   if (SendingInterfaceAddr != INADDR_ANY) {
     struct in_addr addr;
     addr.s_addr = SendingInterfaceAddr;
-    
+
     if (setsockopt(newSocket, IPPROTO_IP, IP_MULTICAST_IF,
 		   (const char*)&addr, sizeof addr) < 0) {
       socketErr(env, "error setting outgoing multicast interface: ");
@@ -128,7 +124,7 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
       return -1;
     }
   }
-  
+
   return newSocket;
 }
 
@@ -145,26 +141,39 @@ Boolean makeSocketNonBlocking(int sock) {
 #endif
 }
 
+Boolean makeSocketBlocking(int sock) {
+#if defined(__WIN32__) || defined(_WIN32) || defined(IMN_PIM)
+  unsigned long arg = 0;
+  return ioctlsocket(sock, FIONBIO, &arg) == 0;
+#elif defined(VXWORKS)
+  int arg = 0;
+  return ioctl(sock, FIONBIO, (int)&arg) == 0;
+#else
+  int curFlags = fcntl(sock, F_GETFL, 0);
+  return fcntl(sock, F_SETFL, curFlags&(~O_NONBLOCK)) >= 0;
+#endif
+}
+
 int setupStreamSocket(UsageEnvironment& env,
                       Port port, Boolean makeNonBlocking) {
   if (!initializeWinsockIfNecessary()) {
     socketErr(env, "Failed to initialize 'winsock': ");
     return -1;
   }
-  
+
   int newSocket = socket(AF_INET, SOCK_STREAM, 0);
   if (newSocket < 0) {
     socketErr(env, "unable to create stream socket: ");
     return newSocket;
   }
-  
+
   if (setsockopt(newSocket, SOL_SOCKET, SO_REUSEADDR,
 		 (const char*)&reuseFlag, sizeof reuseFlag) < 0) {
     socketErr(env, "setsockopt(SO_REUSEADDR) error: ");
     closeSocket(newSocket);
     return -1;
   }
-  
+
   // SO_REUSEPORT doesn't really make sense for TCP sockets, so we
   // normally don't set them.  However, if you really want to do this
   // #define REUSE_FOR_TCP
@@ -223,19 +232,17 @@ static int blockUntilReadable(UsageEnvironment& env,
     if (socket < 0) break;
     FD_SET((unsigned) socket, &rd_set);
     const unsigned numFds = socket+1;
-    
+
     result = select(numFds, &rd_set, NULL, NULL, timeout);
     if (timeout != NULL && result == 0) {
       break; // this is OK - timeout occurred
     } else if (result <= 0) {
-#if defined(__WIN32__) || defined(_WIN32)
-#else
-      if (errno == EINTR || errno == EAGAIN) continue;
-#endif
+      int err = env.getErrno();
+      if (err == EINTR || err == EAGAIN || err == EWOULDBLOCK) continue;
       socketErr(env, "select() error: ");
       break;
     }
-    
+
     if (!FD_ISSET(socket, &rd_set)) {
       socketErr(env, "select() error - !FD_ISSET");
       break;
@@ -262,7 +269,7 @@ int readSocket(UsageEnvironment& env,
     } else if (result <= 0) {
       break;
     }
-    
+
     SOCKLEN_T addressSize = sizeof fromAddress;
     bytesRead = recvfrom(socket, (char*)buffer, bufferSize, 0,
 			 (struct sockaddr*)&fromAddress,
@@ -292,7 +299,7 @@ int readSocket(UsageEnvironment& env,
       break;
     }
   } while (0);
-  
+
   return bytesRead;
 }
 
@@ -303,7 +310,7 @@ int readSocketExact(UsageEnvironment& env,
 		    struct timeval* timeout) {
   /* read EXACTLY bufferSize bytes from the socket into the buffer.
      fromaddress is address of last read.
-     return the number of bytes acually read when an error occurs
+     return the number of bytes actually read when an error occurs
   */
   int bsize = bufferSize;
   int bytesRead = 0;
@@ -440,7 +447,7 @@ Boolean socketJoinGroup(UsageEnvironment& env, int socket,
     }
 #endif
   }
-  
+
   return True;
 }
 
@@ -505,7 +512,7 @@ Boolean socketJoinGroupSSM(UsageEnvironment& env, int socket,
     socketErr(env, "setsockopt(IP_ADD_SOURCE_MEMBERSHIP) error: ");
     return False;
   }
-  
+
   return True;
 }
 
@@ -522,7 +529,7 @@ Boolean socketLeaveGroupSSM(UsageEnvironment& /*env*/, int socket,
 		 (const char*)&imr, sizeof (struct ip_mreq_source)) < 0) {
     return False;
   }
-  
+
   return True;
 }
 
@@ -530,7 +537,7 @@ static Boolean getSourcePort0(int socket, portNumBits& resultPortNum/*host order
   sockaddr_in test; test.sin_port = 0;
   SOCKLEN_T len = sizeof test;
   if (getsockname(socket, (struct sockaddr*)&test, &len) < 0) return False;
-  
+
   resultPortNum = ntohs(test.sin_port);
   return True;
 }
@@ -547,7 +554,7 @@ Boolean getSourcePort(UsageEnvironment& env, int socket, Port& port) {
       return False;
     }
   }
-  
+
   port = Port(portNum);
   return True;
 }
@@ -562,16 +569,16 @@ static Boolean badAddress(netAddressBits addr) {
 
 Boolean loopbackWorks = 1;
 
-netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
+netAddressBits ourIPAddress(UsageEnvironment& env) {
   static netAddressBits ourAddress = 0;
   int sock = -1;
   struct in_addr testAddr;
-  
+
   if (ourAddress == 0) {
     // We need to find our source address
     struct sockaddr_in fromAddr;
     fromAddr.sin_addr.s_addr = 0;
-    
+
     // Get our address by sending a (0-TTL) multicast packet,
     // receiving it, and looking at the source address used.
     // (This is kinda bogus, but it provides the best guarantee
@@ -581,18 +588,18 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
 
       testAddr.s_addr = our_inet_addr("228.67.43.91"); // arbitrary
       Port testPort(15947); // ditto
-      
+
       sock = setupDatagramSocket(env, testPort);
       if (sock < 0) break;
-      
+
       if (!socketJoinGroup(env, sock, testAddr.s_addr)) break;
-      
+
       unsigned char testString[] = "hostIdTest";
       unsigned testStringLength = sizeof testString;
-      
+
       if (!writeSocket(env, sock, testAddr, testPort, 0,
 		       testString, testStringLength)) break;
-      
+
       unsigned char readBuffer[20];
       struct timeval timeout;
       timeout.tv_sec = 5;
@@ -622,7 +629,7 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
 	env.setResultErrMsg("initial gethostname() failed");
 	break;
       }
-      
+
 #if defined(VXWORKS)
 #include <hostLib.h>
       if (ERROR == (ourAddress = hostGetByName( hostname ))) break;
@@ -639,7 +646,7 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
       for (unsigned i = 0; ; ++i) {
 	char* addrPtr = hstent->h_addr_list[i];
 	if (addrPtr == NULL) break;
-	
+
 	netAddressBits a = *(netAddressBits*)addrPtr;
 	if (!badAddress(a)) {
 	  addr = a;
@@ -653,7 +660,7 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
 	break;
       }
     } while (0);
-    
+
     // Make sure we have a good address:
     netAddressBits from = fromAddr.sin_addr.s_addr;
     if (badAddress(from)) {
@@ -664,15 +671,15 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
       env.setResultMsg(tmp);
       from = 0;
     }
-    
+
     ourAddress = from;
 #endif
-    
+
     if (sock >= 0) {
       socketLeaveGroup(env, sock, testAddr.s_addr);
       closeSocket(sock);
     }
-    
+
     // Use our newly-discovered IP address, and the current time,
     // to initialize the random number generator's seed:
     struct timeval timeNow;
@@ -685,7 +692,7 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
 
 netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
   // First, a hack to ensure that our random number generator is seeded:
-  (void) ourSourceAddressForMulticast(env);
+  (void) ourIPAddress(env);
 
   // Choose a random address in the range [232.0.1.0, 232.255.255.255)
   // i.e., [0xE8000100, 0xE8FFFFFF)
@@ -698,7 +705,7 @@ netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
 char const* timestampString() {
   struct timeval tvNow;
   gettimeofday(&tvNow, NULL);
-  
+
 #if !defined(_WIN32_WCE)
   static char timeString[9]; // holds hh:mm:ss plus trailing '\0'
   char const* ctimeResult = ctime((time_t*)&tvNow.tv_sec);
@@ -715,7 +722,7 @@ char const* timestampString() {
   static char timeString[50];
   sprintf(timeString, "%lu.%06ld", tvNow.tv_sec, tvNow.tv_usec);
 #endif
-  
+
   return (char const*)&timeString;
 }
 
@@ -728,7 +735,7 @@ char const* timestampString() {
 int gettimeofday(struct timeval* tp, int* /*tz*/) {
 #if defined(_WIN32_WCE)
   /* FILETIME of Jan 1 1970 00:00:00. */
-  static const unsigned __int64 epoch = 116444736000000000L;
+  static const unsigned __int64 epoch = 116444736000000000LL;
 
   FILETIME    file_time;
   SYSTEMTIME  system_time;
@@ -742,23 +749,36 @@ int gettimeofday(struct timeval* tp, int* /*tz*/) {
   tp->tv_sec = (long) ((ularge.QuadPart - epoch) / 10000000L);
   tp->tv_usec = (long) (system_time.wMilliseconds * 1000);
 #else
-#ifdef USE_OLD_GETTIMEOFDAY_FOR_WINDOWS_CODE
-  struct timeb tb;
-  ftime(&tb);
-  tp->tv_sec = tb.time;
-  tp->tv_usec = 1000*tb.millitm;
-#else
+  static LARGE_INTEGER tickFrequency, epochOffset;
+
+  // For our first call, use "ftime()", so that we get a time with a proper epoch.
+  // For subsequent calls, use "QueryPerformanceCount()", because it's more fine-grain.
+  static Boolean isFirstCall = True;
+
   LARGE_INTEGER tickNow;
-  static LARGE_INTEGER tickFrequency;
-  static BOOL tickFrequencySet = FALSE;
-  if (tickFrequencySet == FALSE) {
-    QueryPerformanceFrequency(&tickFrequency);
-    tickFrequencySet = TRUE;
-  }
   QueryPerformanceCounter(&tickNow);
-  tp->tv_sec = (long) (tickNow.QuadPart / tickFrequency.QuadPart);
-  tp->tv_usec = (long) (((tickNow.QuadPart % tickFrequency.QuadPart) * 1000000L) / tickFrequency.QuadPart);
-#endif
+
+  if (isFirstCall) {
+    struct timeb tb;
+    ftime(&tb);
+    tp->tv_sec = tb.time;
+    tp->tv_usec = 1000*tb.millitm;
+
+    // Also get our counter frequency:
+    QueryPerformanceFrequency(&tickFrequency);
+
+    // And compute an offset to add to subsequent counter times, so we get a proper epoch:
+    epochOffset.QuadPart
+      = tb.time*tickFrequency.QuadPart + (tb.millitm*tickFrequency.QuadPart)/1000 - tickNow.QuadPart;
+
+    isFirstCall = False; // for next time
+  } else {
+    // Adjust our counter time so that we get a proper epoch:
+    tickNow.QuadPart += epochOffset.QuadPart;
+
+    tp->tv_sec = (long) (tickNow.QuadPart / tickFrequency.QuadPart);
+    tp->tv_usec = (long) (((tickNow.QuadPart % tickFrequency.QuadPart) * 1000000L) / tickFrequency.QuadPart);
+  }
 #endif
   return 0;
 }
