@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // A RTSP server
 // Implementation
 
@@ -29,9 +29,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #endif
 #include <time.h> // for "strftime()" and "gmtime()"
 
-#define RTPINFO_INCLUDE_RTPTIME 1
-
-////////// RTSPServer //////////
+////////// RTSPServer implementation //////////
 
 RTSPServer*
 RTSPServer::createNew(UsageEnvironment& env, Port ourPort,
@@ -40,7 +38,7 @@ RTSPServer::createNew(UsageEnvironment& env, Port ourPort,
   int ourSocket = -1;
 
   do {
-    int ourSocket = setUpOurSocket(env, ourPort);
+    ourSocket = setUpOurSocket(env, ourPort);
     if (ourSocket == -1) break;
 
     return new RTSPServer(env, ourSocket, ourPort, authDatabase,
@@ -99,28 +97,34 @@ void RTSPServer::removeServerMediaSession(char const* streamName) {
   removeServerMediaSession(lookupServerMediaSession(streamName));
 }
 
-char* RTSPServer::rtspURLPrefix() const {
-  struct in_addr ourAddress;
-  ourAddress.s_addr = ReceivingInterfaceAddr != 0
-    ? ReceivingInterfaceAddr
-    : ourSourceAddressForMulticast(envir()); // hack
+char* RTSPServer::rtspURLPrefix(int clientSocket) const {
+  struct sockaddr_in ourAddress;
+  if (clientSocket < 0) {
+    // Use our default IP address in the URL:
+    ourAddress.sin_addr.s_addr = ReceivingInterfaceAddr != 0
+      ? ReceivingInterfaceAddr
+      : ourIPAddress(envir()); // hack
+  } else {
+    SOCKLEN_T namelen = sizeof ourAddress;
+    getsockname(clientSocket, (struct sockaddr*)&ourAddress, &namelen);
+  }
 
   char urlBuffer[100]; // more than big enough for "rtsp://<ip-address>:<port>/"
 
   portNumBits portNumHostOrder = ntohs(fServerPort.num());
   if (portNumHostOrder == 554 /* the default port number */) {
-    sprintf(urlBuffer, "rtsp://%s/", our_inet_ntoa(ourAddress));
+    sprintf(urlBuffer, "rtsp://%s/", our_inet_ntoa(ourAddress.sin_addr));
   } else {
     sprintf(urlBuffer, "rtsp://%s:%hu/",
-	    our_inet_ntoa(ourAddress), portNumHostOrder);
+	    our_inet_ntoa(ourAddress.sin_addr), portNumHostOrder);
   }
 
   return strDup(urlBuffer);
 }
 
 char* RTSPServer
-::rtspURL(ServerMediaSession const* serverMediaSession) const {
-  char* urlPrefix = rtspURLPrefix();
+::rtspURL(ServerMediaSession const* serverMediaSession, int clientSocket) const {
+  char* urlPrefix = rtspURLPrefix(clientSocket);
   char const* sessionName = serverMediaSession->streamName();
 
   char* resultURL = new char[strlen(urlPrefix) + strlen(sessionName) + 1];
@@ -154,12 +158,18 @@ int RTSPServer::setUpOurSocket(UsageEnvironment& env, Port& ourPort) {
     }
 
     return ourSocket;
-  } while (0);  
+  } while (0);
 
   if (ourSocket != -1) ::closeSocket(ourSocket);
   return -1;
 }
 
+Boolean RTSPServer
+::specialClientAccessCheck(int /*clientSocket*/, struct sockaddr_in& /*clientAddr*/, char const* /*urlSuffix*/) {
+  // default implementation
+  return True;
+}
+
 RTSPServer::RTSPServer(UsageEnvironment& env,
 		       int ourSocket, Port ourPort,
 		       UserAuthenticationDatabase* authDatabase,
@@ -167,8 +177,7 @@ RTSPServer::RTSPServer(UsageEnvironment& env,
   : Medium(env),
     fServerSocket(ourSocket), fServerPort(ourPort),
     fAuthDB(authDatabase), fReclamationTestSeconds(reclamationTestSeconds),
-    fServerMediaSessions(HashTable::create(STRING_HASH_KEYS)), 
-    fSessionIdCounter(0) {
+    fServerMediaSessions(HashTable::create(STRING_HASH_KEYS)) {
 #ifdef USE_SIGNALS
   // Ignore the SIGPIPE signal, so that clients on the same host that are killed
   // don't also kill us:
@@ -221,17 +230,21 @@ void RTSPServer::incomingConnectionHandler1() {
     return;
   }
   makeSocketNonBlocking(clientSocket);
+  increaseSendBufferTo(envir(), clientSocket, 50*1024);
+
 #if defined(DEBUG) || defined(DEBUG_CONNECTIONS)
-  fprintf(stderr, "accept()ed connection from %s\n", our_inet_ntoa(clientAddr.sin_addr));
+  envir() << "accept()ed connection from " << our_inet_ntoa(clientAddr.sin_addr) << '\n';
 #endif
 
-  // Create a new object for this RTSP session:
-  new RTSPClientSession(*this, ++fSessionIdCounter,
-			clientSocket, clientAddr);
+  // Create a new object for this RTSP session.
+  // (Choose a random 32-bit integer for the session id (it will be encoded as a 8-digit hex number).  We don't bother checking for
+  //  a collision; the probability of two concurrent sessions getting the same session id is very low.)
+  unsigned sessionId = (unsigned)our_random();
+  (void)createNewClientSession(sessionId, clientSocket, clientAddr);
 }
 
 
-////////// RTSPServer::RTSPClientSession //////////
+////////// RTSPServer::RTSPClientSession implementation //////////
 
 RTSPServer::RTSPClientSession
 ::RTSPClientSession(RTSPServer& ourServer, unsigned sessionId,
@@ -298,7 +311,7 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
   struct sockaddr_in dummy; // 'from' address, meaningless in this case
   Boolean endOfMsg = False;
   unsigned char* ptr = &fRequestBuffer[fRequestBytesAlreadySeen];
-  
+
   int bytesRead = readSocket(envir(), fClientSocket,
 			     ptr, fRequestBufferBytesLeft, dummy);
   if (bytesRead <= 0 || (unsigned)bytesRead >= fRequestBufferBytesLeft) {
@@ -329,7 +342,7 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
     }
     ++tmpPtr;
   }
-  
+
   fRequestBufferBytesLeft -= bytesRead;
   fRequestBytesAlreadySeen += bytesRead;
 
@@ -364,14 +377,15 @@ void RTSPServer::RTSPClientSession::incomingRequestHandler1() {
     } else if (strcmp(cmdName, "TEARDOWN") == 0
 	       || strcmp(cmdName, "PLAY") == 0
 	       || strcmp(cmdName, "PAUSE") == 0
-	       || strcmp(cmdName, "GET_PARAMETER") == 0) {
+	       || strcmp(cmdName, "GET_PARAMETER") == 0
+	       || strcmp(cmdName, "SET_PARAMETER") == 0) {
       handleCmd_withinSession(cmdName, urlPreSuffix, urlSuffix, cseq,
 			      (char const*)fRequestBuffer);
     } else {
       handleCmd_notSupported(cseq);
     }
   }
-    
+
 #ifdef DEBUG
   fprintf(stderr, "sending response: %s", fResponseBuffer);
 #endif
@@ -406,7 +420,7 @@ static char const* dateHeader() {
   WCHAR timeFormat[] = L"HH:mm:ss GMT\r\n";
   WCHAR inBuf[200];
   DWORD locale = LOCALE_NEUTRAL;
-  
+
   int ret = GetDateFormat(locale, 0, &SystemTime,
 			  (LPTSTR)dateFormat, (LPTSTR)inBuf, sizeof inBuf);
   inBuf[ret - 1] = ' ';
@@ -419,21 +433,20 @@ static char const* dateHeader() {
 }
 
 static char const* allowedCommandNames
-  = "OPTIONS, DESCRIBE, SETUP, TEARDOWN, PLAY, PAUSE";
+//  = "OPTIONS, DESCRIBE, SETUP, TEARDOWN, PLAY, PAUSE, GET_PARAMETER, SET_PARAMETER";
+= "OPTIONS, DESCRIBE, SETUP, TEARDOWN, PLAY, PAUSE, SET_PARAMETER"; // TEMP HACK to stop VLC from using "GET_PARAMETER" as a client 'keep-alive' indicator; we don't need this, and it currently causes problems for RTP-over-TCP streams.
 
 void RTSPServer::RTSPClientSession::handleCmd_bad(char const* /*cseq*/) {
   // Don't do anything with "cseq", because it might be nonsense
   snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 400 Bad Request\r\n%sAllow: %s\r\n\r\n",
 	   dateHeader(), allowedCommandNames);
-  fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_notSupported(char const* cseq) {
   snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 405 Method Not Allowed\r\nCSeq: %s\r\n%sAllow: %s\r\n\r\n",
 	   cseq, dateHeader(), allowedCommandNames);
-  fSessionIsActive = False; // triggers deletion of ourself after responding
 }
 
 void RTSPServer::RTSPClientSession::handleCmd_notFound(char const* cseq) {
@@ -462,7 +475,8 @@ void RTSPServer::RTSPClientSession
   char* sdpDescription = NULL;
   char* rtspURL = NULL;
   do {
-    if (!authenticationOK("DESCRIBE", cseq, fullRequestStr)) break;
+      if (!authenticationOK("DESCRIBE", cseq, urlSuffix, fullRequestStr))
+          break;
 
     // We should really check that the request contains an "Accept:" #####
     // for "application/sdp", because that's what we're sending back #####
@@ -493,7 +507,7 @@ void RTSPServer::RTSPClientSession
     // Also, generate our RTSP URL, for the "Content-Base:" header
     // (which is necessary to ensure that the correct URL gets used in
     // subsequent "SETUP" requests).
-    rtspURL = fOurServer.rtspURL(session);
+    rtspURL = fOurServer.rtspURL(session, fClientSocket);
 
     snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	     "RTSP/1.0 200 OK\r\nCSeq: %s\r\n"
@@ -517,7 +531,7 @@ typedef enum StreamingMode {
   RTP_UDP,
   RTP_TCP,
   RAW_UDP
-};
+} StreamingMode;
 
 static void parseTransportHeader(char const* buf,
 				 StreamingMode& streamingMode,
@@ -535,7 +549,7 @@ static void parseTransportHeader(char const* buf,
   destinationAddressStr = NULL;
   destinationTTL = 255;
   clientRTPPortNum = 0;
-  clientRTCPPortNum = 1; 
+  clientRTCPPortNum = 1;
   rtpChannelId = rtcpChannelId = 0xFF;
 
   portNumBits p1, p2;
@@ -581,33 +595,6 @@ static void parseTransportHeader(char const* buf,
   delete[] field;
 }
 
-static Boolean parseRangeHeader(char const* buf, float& rangeStart, float& rangeEnd) {
-  // Initialize the result parameters to default values:
-  rangeStart = rangeEnd = 0.0;
-
-  // First, find "Range:"
-  while (1) {
-    if (*buf == '\0') return False; // not found
-    if (_strncasecmp(buf, "Range: ", 7) == 0) break;
-    ++buf;
-  }
-
-  // Then, run through each of the fields, looking for ones we handle:
-  char const* fields = buf + 7;
-  while (*fields == ' ') ++fields;
-  float start, end;
-  if (sscanf(fields, "npt = %f - %f", &start, &end) == 2) {
-    rangeStart = start;
-    rangeEnd = end;
-  } else if (sscanf(fields, "npt = %f -", &start) == 1) {
-    rangeStart = start;
-  } else {
-    return False; // The header is malformed
-  }
-
-  return True;
-}
-
 static Boolean parsePlayNowHeader(char const* buf) {
   // Find "x-playNow:" header, if present
   while (1) {
@@ -695,7 +682,7 @@ void RTSPServer::RTSPClientSession
   // Look for a "Transport:" header in the request string,
   // to extract client parameters:
   StreamingMode streamingMode;
-  char* streamingModeString; // set when RAW_UDP streaming is specified
+  char* streamingModeString = NULL; // set when RAW_UDP streaming is specified
   char* clientsDestinationAddressStr;
   u_int8_t clientsDestinationTTL;
   portNumBits clientRTPPortNum, clientRTCPPortNum;
@@ -717,7 +704,7 @@ void RTSPServer::RTSPClientSession
 
   // Next, check whether a "Range:" header is present in the request.
   // This isn't legal, but some clients do this to combine "SETUP" and "PLAY":
-  float rangeStart, rangeEnd;
+  double rangeStart = 0.0, rangeEnd = 0.0;
   fStreamAfterSETUP = parseRangeHeader(fullRequestStr, rangeStart, rangeEnd) ||
                       parsePlayNowHeader(fullRequestStr);
 
@@ -746,6 +733,10 @@ void RTSPServer::RTSPClientSession
 				  serverRTPPort, serverRTCPPort,
 				  fStreamStates[streamNum].streamToken);
   struct in_addr destinationAddr; destinationAddr.s_addr = destinationAddress;
+  char* destAddrStr = strDup(our_inet_ntoa(destinationAddr));
+  struct sockaddr_in sourceAddr; SOCKLEN_T namelen = sizeof sourceAddr;
+  getsockname(fClientSocket, (struct sockaddr*)&sourceAddr, &namelen);
+  char* sourceAddrStr = strDup(our_inet_ntoa(sourceAddr.sin_addr));
   if (fIsMulticast) {
     switch (streamingMode) {
     case RTP_UDP:
@@ -753,11 +744,11 @@ void RTSPServer::RTSPClientSession
                  "RTSP/1.0 200 OK\r\n"
                  "CSeq: %s\r\n"
                  "%s"
-                 "Transport: RTP/AVP;multicast;destination=%s;port=%d-%d;ttl=%d\r\n"
-                 "Session: %d\r\n\r\n",
+                 "Transport: RTP/AVP;multicast;destination=%s;source=%s;port=%d-%d;ttl=%d\r\n"
+                 "Session: %08X\r\n\r\n",
                  cseq,
                  dateHeader(),
-                 our_inet_ntoa(destinationAddr), ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()), destinationTTL,
+                 destAddrStr, sourceAddrStr, ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()), destinationTTL,
                  fOurSessionId);
         break;
     case RTP_TCP:
@@ -769,15 +760,12 @@ void RTSPServer::RTSPClientSession
                  "RTSP/1.0 200 OK\r\n"
                  "CSeq: %s\r\n"
                  "%s"
-                 "Transport: %s;multicast;destination=%s;client_port=%d;server_port=%d\r\n"
-                 "Session: %d\r\n\r\n",
+                 "Transport: %s;multicast;destination=%s;source=%s;port=%d;ttl=%d\r\n"
+                 "Session: %08X\r\n\r\n",
                  cseq,
                  dateHeader(),
-                 streamingModeString,
-                 our_inet_ntoa(destinationAddr),
-                 ntohs(clientRTPPort.num()), ntohs(serverRTPPort.num()),
+                 streamingModeString, destAddrStr, sourceAddrStr, ntohs(serverRTPPort.num()), destinationTTL,
                  fOurSessionId);
-        delete[] streamingModeString;
         break;
     }
   } else {
@@ -787,11 +775,11 @@ void RTSPServer::RTSPClientSession
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
-	       "Transport: RTP/AVP;unicast;destination=%s;client_port=%d-%d;server_port=%d-%d\r\n"
-	       "Session: %d\r\n\r\n",
+	       "Transport: RTP/AVP;unicast;destination=%s;source=%s;client_port=%d-%d;server_port=%d-%d\r\n"
+	       "Session: %08X\r\n\r\n",
 	       cseq,
 	       dateHeader(),
-	       our_inet_ntoa(destinationAddr), ntohs(clientRTPPort.num()), ntohs(clientRTCPPort.num()), ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()),
+	       destAddrStr, sourceAddrStr, ntohs(clientRTPPort.num()), ntohs(clientRTCPPort.num()), ntohs(serverRTPPort.num()), ntohs(serverRTCPPort.num()),
 	       fOurSessionId);
       break;
     }
@@ -800,11 +788,11 @@ void RTSPServer::RTSPClientSession
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
-	       "Transport: RTP/AVP/TCP;unicast;destination=%s;interleaved=%d-%d\r\n"
-	       "Session: %d\r\n\r\n",
+	       "Transport: RTP/AVP/TCP;unicast;destination=%s;source=%s;interleaved=%d-%d\r\n"
+	       "Session: %08X\r\n\r\n",
 	       cseq,
 	       dateHeader(),
-	       our_inet_ntoa(destinationAddr), rtpChannelId, rtcpChannelId,
+	       destAddrStr, sourceAddrStr, rtpChannelId, rtcpChannelId,
 	       fOurSessionId);
       break;
     }
@@ -813,17 +801,17 @@ void RTSPServer::RTSPClientSession
 	       "RTSP/1.0 200 OK\r\n"
 	       "CSeq: %s\r\n"
 	       "%s"
-	       "Transport: %s;unicast;destination=%s;client_port=%d;server_port=%d\r\n"
-	       "Session: %d\r\n\r\n",
+	       "Transport: %s;unicast;destination=%s;source=%s;client_port=%d;server_port=%d\r\n"
+	       "Session: %08X\r\n\r\n",
 	       cseq,
 	       dateHeader(),
-	       streamingModeString, our_inet_ntoa(destinationAddr), ntohs(clientRTPPort.num()), ntohs(serverRTPPort.num()),
+	       streamingModeString, destAddrStr, sourceAddrStr, ntohs(clientRTPPort.num()), ntohs(serverRTPPort.num()),
 	       fOurSessionId);
-      delete[] streamingModeString;
       break;
     }
     }
   }
+  delete[] destAddrStr; delete[] sourceAddrStr; delete[] streamingModeString;
 }
 
 void RTSPServer::RTSPClientSession
@@ -871,6 +859,8 @@ void RTSPServer::RTSPClientSession
     handleCmd_PAUSE(subsession, cseq);
   } else if (strcmp(cmdName, "GET_PARAMETER") == 0) {
     handleCmd_GET_PARAMETER(subsession, cseq, fullRequestStr);
+  } else if (strcmp(cmdName, "SET_PARAMETER") == 0) {
+    handleCmd_SET_PARAMETER(subsession, cseq, fullRequestStr);
   }
 }
 
@@ -909,10 +899,10 @@ static Boolean parseScaleHeader(char const* buf, float& scale) {
 void RTSPServer::RTSPClientSession
   ::handleCmd_PLAY(ServerMediaSubsession* subsession, char const* cseq,
 		   char const* fullRequestStr) {
-  char* rtspURL = fOurServer.rtspURL(fOurServerMediaSession);
+  char* rtspURL = fOurServer.rtspURL(fOurServerMediaSession, fClientSocket);
   unsigned rtspURLSize = strlen(rtspURL);
 
-  //// Parse the client's "Scale:" header, if any: 
+  // Parse the client's "Scale:" header, if any:
   float scale;
   Boolean sawScaleHeader = parseScaleHeader(fullRequestStr, scale);
 
@@ -932,8 +922,8 @@ void RTSPServer::RTSPClientSession
   }
   scaleHeader = strDup(buf);
 
-  //// Parse the client's "Range:" header, if any: 
-  float rangeStart, rangeEnd;
+  // Parse the client's "Range:" header, if any:
+  double rangeStart = 0.0, rangeEnd = 0.0;
   Boolean sawRangeHeader = parseRangeHeader(fullRequestStr, rangeStart, rangeEnd);
 
   // Use this information, plus the stream's duration (if known), to create
@@ -946,7 +936,7 @@ void RTSPServer::RTSPClientSession
     duration = -duration;
   }
 
-  if (rangeEnd < 0.0 || rangeEnd > duration) rangeEnd = duration;
+  if (rangeEnd <= 0.0 || rangeEnd > duration) rangeEnd = duration;
   if (rangeStart < 0.0) {
     rangeStart = 0.0;
   } else if (rangeEnd > 0.0 && scale > 0.0 && rangeStart > rangeEnd) {
@@ -966,12 +956,10 @@ void RTSPServer::RTSPClientSession
   // Create a "RTP-Info:" line.  It will get filled in from each subsession's state:
   char const* rtpInfoFmt =
     "%s" // "RTP-Info:", plus any preceding rtpInfo items
-    "%s" // comma separator, if needed 
+    "%s" // comma separator, if needed
     "url=%s/%s"
     ";seq=%d"
-#ifdef RTPINFO_INCLUDE_RTPTIME
     ";rtptime=%u"
-#endif
     ;
   unsigned rtpInfoFmtSize = strlen(rtpInfoFmt);
   char* rtpInfo = strDup("RTP-Info: ");
@@ -1013,19 +1001,15 @@ void RTSPServer::RTSPClientSession
 	+ 1
 	+ rtspURLSize + strlen(urlSuffix)
 	+ 5 /*max unsigned short len*/
-#ifdef RTPINFO_INCLUDE_RTPTIME
 	+ 10 /*max unsigned (32-bit) len*/
-#endif
-	+ 2 /*allows for trailing \r\n at final end of string*/; 
+	+ 2 /*allows for trailing \r\n at final end of string*/;
       rtpInfo = new char[rtpInfoSize];
       sprintf(rtpInfo, rtpInfoFmt,
 	      prevRTPInfo,
 	      numRTPInfoItems++ == 0 ? "" : ",",
 	      rtspURL, urlSuffix,
-	      rtpSeqNum
-#ifdef RTPINFO_INCLUDE_RTPTIME
-	      ,rtpTimestamp
-#endif
+	      rtpSeqNum,
+	      rtpTimestamp
 	      );
       delete[] prevRTPInfo;
     }
@@ -1046,7 +1030,7 @@ void RTSPServer::RTSPClientSession
 	   "%s"
 	   "%s"
 	   "%s"
-	   "Session: %d\r\n"
+	   "Session: %08X\r\n"
 	   "%s\r\n",
 	   cseq,
 	   dateHeader(),
@@ -1068,7 +1052,7 @@ void RTSPServer::RTSPClientSession
     }
   }
   snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
-	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %d\r\n\r\n",
+	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %08X\r\n\r\n",
 	   cseq, dateHeader(), fOurSessionId);
 }
 
@@ -1078,10 +1062,17 @@ void RTSPServer::RTSPClientSession
   // We implement "GET_PARAMETER" just as a 'keep alive',
   // and send back an empty response:
   snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
-	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %d\r\n\r\n",
+	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %08X\r\n\r\n",
 	   cseq, dateHeader(), fOurSessionId);
 }
 
+void RTSPServer::RTSPClientSession
+::handleCmd_SET_PARAMETER(ServerMediaSubsession* /*subsession*/, char const* cseq,
+			  char const* /*fullRequestStr*/) {
+  // By default, we don't implement "SET_PARAMETER":
+  handleCmd_notSupported(cseq);
+}
+
 static Boolean parseAuthorizationHeader(char const* buf,
 					char const*& username,
 					char const*& realm,
@@ -1131,7 +1122,18 @@ static Boolean parseAuthorizationHeader(char const* buf,
 
 Boolean RTSPServer::RTSPClientSession
 ::authenticationOK(char const* cmdName, char const* cseq,
-		   char const* fullRequestStr) {
+		   char const* urlSuffix, char const* fullRequestStr) {
+
+  if (!fOurServer.specialClientAccessCheck(fClientSocket, fClientAddr, urlSuffix)) {
+    snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+             "RTSP/1.0 401 Unauthorized\r\n"
+             "CSeq: %s\r\n"
+             "%s"
+             "\r\n",
+             cseq, dateHeader());
+    return False;
+  }
+
   // If we weren't set up with an authentication database, we're OK:
   if (fOurServer.fAuthDB == NULL) return True;
 
@@ -1210,18 +1212,37 @@ void RTSPServer::RTSPClientSession
 ::livenessTimeoutTask(RTSPClientSession* clientSession) {
   // If this gets called, the client session is assumed to have timed out,
   // so delete it:
-
-  // However, we don't timeout multicast sessions, because to do so would require
-  // closing all client sessions that have requested the stream - not just this one.
-  // Also, the multicast stream itself would usually not be halted, in any case.
-  if (clientSession->isMulticast()) return;
-
 #ifdef DEBUG
   fprintf(stderr, "RTSP client session from %s has timed out (due to inactivity)\n", our_inet_ntoa(clientSession->fClientAddr.sin_addr));
 #endif
   delete clientSession;
 }
 
+RTSPServer::RTSPClientSession*
+RTSPServer::createNewClientSession(unsigned sessionId, int clientSocket, struct sockaddr_in clientAddr) {
+  return new RTSPClientSession(*this, sessionId, clientSocket, clientAddr);
+}
+
+
+////////// ServerMediaSessionIterator implementation //////////
+
+RTSPServer::ServerMediaSessionIterator
+::ServerMediaSessionIterator(RTSPServer& server)
+  : fOurIterator((server.fServerMediaSessions == NULL)
+		 ? NULL : HashTable::Iterator::create(*server.fServerMediaSessions)) {
+}
+
+RTSPServer::ServerMediaSessionIterator::~ServerMediaSessionIterator() {
+  delete fOurIterator;
+}
+
+ServerMediaSession* RTSPServer::ServerMediaSessionIterator::next() {
+  if (fOurIterator == NULL) return NULL;
+
+  char const* key; // dummy
+  return (ServerMediaSession*)(fOurIterator->next(key));
+}
+
 
 ////////// UserAuthenticationDatabase implementation //////////
 
