@@ -11,14 +11,15 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
 // LIVE555 Media Server
 // main program
 
 #include <BasicUsageEnvironment.hh>
 #include "DynamicRTSPServer.hh"
+#include "RTSPOverHTTPServer.hh"
 #include "version.hh"
 
 int main(int argc, char** argv) {
@@ -38,8 +39,12 @@ int main(int argc, char** argv) {
   // Create the RTSP server.  Try first with the default port number (554),
   // and then with the alternative port number (8554):
   RTSPServer* rtspServer;
-  rtspServer = DynamicRTSPServer::createNew(*env, 554, authDB);
-  if (rtspServer == NULL) rtspServer = DynamicRTSPServer::createNew(*env, 8554, authDB);
+  portNumBits rtspServerPortNum = 554;
+  rtspServer = DynamicRTSPServer::createNew(*env, rtspServerPortNum, authDB);
+  if (rtspServer == NULL) {
+    rtspServerPortNum = 8554;
+    rtspServer = DynamicRTSPServer::createNew(*env, rtspServerPortNum, authDB);
+  }
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
     exit(1);
@@ -57,6 +62,7 @@ int main(int argc, char** argv) {
   *env << "\t\".aac\" => an AAC Audio (ADTS format) file\n";
   *env << "\t\".amr\" => an AMR Audio file\n";
   *env << "\t\".m4e\" => a MPEG-4 Video Elementary Stream file\n";
+  *env << "\t\".dv\" => a DV Video file\n";
   *env << "\t\".mp3\" => a MPEG-1 or 2 Audio file\n";
   *env << "\t\".mpg\" => a MPEG-1 or 2 Program Stream (audio+video) file\n";
   *env << "\t\".ts\" => a MPEG Transport Stream file\n";
@@ -64,6 +70,24 @@ int main(int argc, char** argv) {
   *env << "\t\".wav\" => a WAV Audio file\n";
   *env << "See http://www.live555.com/mediaServer/ for additional documentation.\n";
 
+#if 0 // RTSP-over-HTTP tunneling is not yet working
+  // Also, attempt to create a HTTP server for RTSP-over-HTTP tunneling.
+  // Try first with the default HTTP port (80), and then with the alternative HTTP
+  // port number (8000).
+  RTSPOverHTTPServer* rtspOverHTTPServer;
+  portNumBits httpServerPortNum = 80;
+  rtspOverHTTPServer = RTSPOverHTTPServer::createNew(*env, httpServerPortNum, rtspServerPortNum);
+  if (rtspOverHTTPServer == NULL) {
+    httpServerPortNum = 8000;
+    rtspOverHTTPServer = RTSPOverHTTPServer::createNew(*env, httpServerPortNum, rtspServerPortNum);
+  }
+  if (rtspOverHTTPServer == NULL) {
+    *env << "(No server for RTSP-over-HTTP tunneling was created.)\n";
+  } else {
+    *env << "(We use port " << httpServerPortNum << " for RTSP-over-HTTP tunneling.)\n";
+  }
+#endif
+
   env->taskScheduler().doEventLoop(); // does not return
 
   return 0; // only to prevent compiler warning
